
/********************************************************************************************************************************************************/
/*                                                                                                                                                      */
/*                                                     PAGE STATS                                                                                       */
/*                                                                                                                                                      */
/********************************************************************************************************************************************************/

/***********************************************************************************/
/* Fonction : 	updateStats                                                        */
/* Paramètres : aucun                                                              */
/* Retour : 	aucun                                                              */
/* Rôle : 		met les statistiques à jour (tableau et graphique) en filtrant les */
/*        		données du xml pour n'avoir que celles demandées par l'utilisateur */
/*   			et en créer un nouveau par auteur selon les regroupements. Crée    */
/*				dynamiquement le tableau et le graphique si c'est demandé par      */
/*				l'utilisaateur.                                                    */
/***********************************************************************************/

function updateStats(){
	// variable pour les requête jQuery réutilisées
	// utilise les contrôles pour les options dans l'interface
	inputdebut = jQuery('#debut').val();
	inputfin = jQuery('#fin').val();
	divresultats = jQuery('#resultats');
	divresgraphique = jQuery('#resgraphique');
	inputtoutesdates = jQuery('#toutesdates').is(':checked');
	inputannuel = jQuery('#annuel').is(':checked');
	inputmensuel = jQuery('#mensuel').is(':checked');
	inputhebdo = jQuery('#hebdo').is(':checked');
	inputtableau = jQuery('#tableau').is(':checked');
	inputgraphique = jQuery('#graphique').is(':checked');
	labelx = new Array();
	dataseries = new Array();
	lescouleurs = new Array();

	// referme les options au moment d'afficher les statistiques
	jQuery('.filtre').find('ul').slideUp();
	jQuery('.showhide').removeClass('fermer');

	// filtre des articles en fonction des utilisateurs sélectionnés 
	// et en fonction de la date de début et de la date de fin si toutes les valeurs ne sont pas considérées
	articles = jQuery('article', xmlarticles).filter(function(index) {
		//id = jQuery(this).find('auteur').attr('id');
		dtpub = jQuery(this).find('datepub').find('texte').text();
		if (inputtoutesdates){
			return true;
		}
		else{
			return dtpub >= inputdebut && dtpub <= inputfin;
		}
	});

	// construction d'un xml d'auteurs pour le traitement, en fonction du filtre
	// pour consulter la structure (qui change en fonction des regroupements), faire un console.log(articles) à la fin du .each
	xmlresultats = jQuery.parseXML("<auteurs></auteurs>");
	articles.each(function(){
		t = jQuery(this);
		id = t.find('auteur').attr('id');
		auteurcherche = jQuery('auteur[id=' + id + ']', xmlresultats);

		// si l'auteur à ajouter n'est pas déjà dans le xml, on l'ajoute et on lui met 1 article. 
		// s'il est déjà présent, on ajoute 1 à son nombre d'articles
		if (auteurcherche.length == 0){
			xmlitem = jQuery.parseXML('<auteur id="' + id + '"><nom>' + t.find('auteur').find('nom').text() + '</nom><prenom>' + t.find('auteur').find('prenom').text() + '</prenom><nomutilisateur>' + t.find('auteur').find('nomutilisateur').text() + '</nomutilisateur><nbarticles total="1"><semaine id="' + t.find('datepub').find('semaine').text() + '">1</semaine><mois id="' + t.find('datepub').find('mois').text() + '">1</mois><annee id="' + t.find('datepub').find('annee').text() + '">1</annee></nbarticles></auteur>');
			jQuery(xmlresultats).children(0).append(jQuery(xmlitem).children(0));
		}
		else{
			// update du nombre total d'articles (attribut total de l'élément nbarticles)
			tagpub =  t.find('datepub');
			auteurcherche.find('nbarticles').attr('total', parseInt(auteurcherche.find('nbarticles').attr('total'), 10) + 1);

			// les 4 blocs suivants fonctionnent de la même façon, par exemple pour l'année : 
			// ajoute un élément <annee id="2014">1</annee> si l'élément avec le id 2014 n'existe pas
			// si l'élément <annee id="2014"> existe, on ajoute 1 au nombre d'articles

			// update du nombre par année
			tagannee = jQuery('annee[id=' + tagpub.find('annee').text()  + ']', auteurcherche.find('nbarticles'));
			if (tagannee.length == 0){
				xmlitem = jQuery.parseXML('<annee id="' + tagpub.find('annee').text() + '">1</annee>');
				jQuery(auteurcherche.find('nbarticles')).append(jQuery(xmlitem).children(0));
			}
			else{
				tagannee.text(parseInt(tagannee.text(), 10) + 1);
			}

			// update du nombre par mois
			tagmois = jQuery('mois[id=' + tagpub.find('mois').text()  + ']', auteurcherche.find('nbarticles'));
			if (tagmois.length == 0){
				xmlitem = jQuery.parseXML('<mois id="' + tagpub.find('mois').text() + '">1</mois>');
				jQuery(auteurcherche.find('nbarticles')).append(jQuery(xmlitem).children(0));
			}
			else{
				tagmois.text(parseInt(tagmois.text(), 10) + 1);
			}

			// update du nombre par semaine
			tagsemaine = jQuery('semaine[id=' + tagpub.find('semaine').text()  + ']', auteurcherche.find('nbarticles'));
			if (tagsemaine.length == 0){
				xmlitem = jQuery.parseXML('<semaine id="' + tagpub.find('semaine').text() + '">1</semaine>');
				jQuery(auteurcherche.find('nbarticles')).append(jQuery(xmlitem).children(0));
			}
			else{
				tagsemaine.text(parseInt(tagsemaine.text(), 10) + 1);
			}
		}
	});

	// tri selon l'entête de colonne (le xml à trier, le nom des balises à chercher, la balise pour faire le tri à l'intérieur de l'autre, l'ordre et le type)
	xmlauteurs = sortXML(xmlresultats, 'auteur', tritag, triordre, tritype);

	// génère le tableau et les entêtes
	// le code inclut la création du tableau et certains calculs, la variable txt contient contient donc le code pour afficher le tableau même s'il n'est pas coché (plutôt que refaire les calculs)
	txt = '<table class="widefat fixed"><tr>';

	// les noms ne sont pas affichés
	txt += '<th scope="col" id="nom" class="manage-column column-title">Nom</span></th>';
	txt += '<th scope="col" id="nbarticles" class="manage-column column-title ">Nombre total d\'articles</th>';

	// s'il y a un filtre pour les dates, génération des entêtes pour le filtre et en fonction des périodes
	if (!inputtoutesdates){
		filtredebut = new Date(parseInt(inputdebut.substr(0, 4), 10), parseInt(inputdebut.substr(5, 2), 10) - 1, parseInt(inputdebut.substr(8, 2), 10))
		filtrefin = new Date(parseInt(inputfin.substr(0, 4), 10), parseInt(inputfin.substr(5, 2), 10) - 1, parseInt(inputfin.substr(8, 2), 10))

		// regroupement : génération des entêtes pour toutes les années entre le début et la fin + étiquettes des X pour le graphique
		if (inputannuel){
			datetemp = filtredebut;
			while (datetemp.getFullYear() <= filtrefin.getFullYear()){
				i = datetemp.getFullYear();
				txt += '<th scope="col">' + i + '</th>';
				labelx.push(i);
				totalannee[i] = 0;
				datetemp.setFullYear(i + 1);
			}
		}
		else if (inputmensuel){
			datetemp = filtredebut;
			while (datetemp <= filtrefin){
				i = arrmois[datetemp.getMonth()] + ' ' + datetemp.getFullYear();
				txt += '<th scope="col">' + i + '</th>';
				labelx.push(i);
				i = datetemp.getFullYear() + '-' + (datetemp.getMonth()+1);
				totalmois[i] = 0;
				datetemp.setMonth(datetemp.getMonth() + 1);
			}
		}
		else if (inputhebdo){
			datetemp = filtredebut.getLundi();
			while (datetemp <= filtrefin){
				i = datetemp.getLundi();
				txt += '<th scope="col">\'' + i.getFullYear().toString().substr(2,2) + '<br>' + arrmoiscourts[i.getMonth()] + '<br>' + i.getDate() + '</th>';
				i = datetemp.getLundi().toYMD();
				labelx.push(i);
				totalsemaine[i] = 0;
				datetemp.setDate(datetemp.getDate() + 7);
			}
		}
	}
	txt += '</tr>';

	// pour chaque auteur (élève), génère une ligne du tableau
	totaltout = 0;
	xmlauteurs.each(function(eleveid){
		t = jQuery(this);
		datanb = new Array();
		totalauteur = parseInt(t.find('nbarticles').attr('total'), 10);

		// les noms ne sont pas affichés sauf celui de l'utilisateur connecté
		if (t.find('nomutilisateur').text() == current){
			txt += '<tr class="current"><td>Moi!</td><td>' + totalauteur + '</td>';
		}
		else{
			txt += '<tr><td>Élève' + (eleveid+1) + '</td><td>' + totalauteur + '</td>';
		}
		totaltout += totalauteur;

		// si des des dates sont précisées (pas de regroupement pour 'toutes les dates')
		if (!inputtoutesdates){
			// crée une date à partir des données dans le champ #debut
			filtredebut = new Date(parseInt(inputdebut.substr(0, 4), 10), parseInt(inputdebut.substr(5, 2), 10) - 1, parseInt(inputdebut.substr(8, 2), 10))

			// regroupement : annuel
			if (inputannuel){
				datetemp = filtredebut;
				while (datetemp.getFullYear() <= filtrefin.getFullYear()){
					i = datetemp.getFullYear();
					nb = jQuery('annee[id=' + i + ']', t.find('nbarticles')).text();
					nb = (isNaN(parseInt(nb, 10)) ? '&nbsp;' : parseInt(nb, 10));
					txt += '<td>' + nb + '&nbsp;</td>';
					nb = (isNaN(parseInt(nb, 10)) ? 0 : parseInt(nb, 10));
					datanb.push(nb);
					totalannee[i] += nb;
					datetemp.setFullYear(i + 1);
				}
			}
			// regroupement : mensuel
			else if (inputmensuel){
				datetemp = filtredebut;
				while (datetemp <= filtrefin){
					i = datetemp.getFullYear() + '-' + (datetemp.getMonth()+1);
					nb = jQuery('mois[id=' + i + ']', t.find('nbarticles')).text()
					txt += '<td>' + nb + '&nbsp;</td>';
					nb = (isNaN(parseInt(nb, 10)) ? 0 : parseInt(nb, 10));
					nb = (isNaN(parseInt(nb, 10)) ? '&nbsp;' : parseInt(nb, 10));
					datanb.push(nb);
					totalmois[i] += nb;
					datetemp.setMonth(datetemp.getMonth() + 1);
				}
			}
			// regroupement : hebdomadaire
			else if (inputhebdo){
				datetemp = filtredebut.getLundi();
				while (datetemp <= filtrefin){
					i = datetemp.toYMD();
					nb = jQuery('semaine[id=' + i + ']', t.find('nbarticles')).text();
					nb = (isNaN(parseInt(nb, 10)) ? '&nbsp;' : parseInt(nb, 10));
					txt += '<td>' + nb + '&nbsp;</td>';
					nb = (isNaN(parseInt(nb, 10)) ? 0 : parseInt(nb, 10));
					datanb.push(nb);
					totalsemaine[i] += nb;
					datetemp.setDate(datetemp.getDate() + 7);
				}
			}
			// avec filtre, mais sans regroupement
			else{
				// les noms ne sont pas affichés
				if (t.find('nomutilisateur').text() == current){
					labelx.push('Moi!');
				}
				else{
					labelx.push('Élève' + (eleveid+1))
				}	
			}
		}
		// toutes les dates
		else{
			// les noms ne sont pas affichés
			if (t.find('nomutilisateur').text() == current){
				labelx.push('Moi!');
			}
			else{
				labelx.push('Élève' + (eleveid+1))
			}
		}
		txt += '</tr>';

		// si c'est un graphique en histogramme (toutes les dates, ou dates filtrées sans regroupement)
		if (datanb.length == 0){
			dataseries.push(totalauteur);
		}
		// sinon, c'est donc un graphique de type ligne (dates filtrées avec regroupements)
		else{
			couleurtemp = '#888888';
			nomtemp = 'Élève' + (eleveid+1);
			if (t.find('nomutilisateur').text() == current){
				nomtemp = 'Moi!';
				couleurtemp = '#a00000';
			}
			objtemp = {name: nomtemp, data: datanb};
			dataseries.push(objtemp);
			lescouleurs.push(couleurtemp);
		}
	});

	// génère la ligne total avec minimalement "Total" et le nombre total, avec en plus le total par période s'il y a des regroupements
	txt += '<tr><th>Total</th>';
	txt += '<th>' + totaltout + '</th>';

	if (!inputtoutesdates){
		filtredebut = new Date(parseInt(inputdebut.substr(0, 4), 10), parseInt(inputdebut.substr(5, 2), 10) - 1, parseInt(inputdebut.substr(8, 2), 10))

		// regroupement : annuel (génération des totaux pour toutes les années entre le début et la fin)			
		if (inputannuel){
			datetemp = filtredebut;
			while (datetemp.getFullYear() <= filtrefin.getFullYear()){
				i = datetemp.getFullYear();
				nb = totalannee[i];
				txt += '<th scope="col">' + nb + '</th>';
				datetemp.setFullYear(i + 1);
			}
		}
		// regroupement : mensuel (génération des totaux pour tous les mois entre le début et la fin)
		else if (inputmensuel){
			datetemp = filtredebut;
			while (datetemp <= filtrefin){
				i = datetemp.getFullYear() + '-' + (datetemp.getMonth()+1);
				nb = totalmois[i];
				txt += '<th scope="col">' + nb + '</th>';
				datetemp.setMonth(datetemp.getMonth() + 1);
			}
		}
		// regroupement : hebdomadaire (génération des totaux pour toutes les semaines entre le début et la fin)
		else if (inputhebdo){
			datetemp = filtredebut.getLundi();
			while (datetemp <= filtrefin){
				i = datetemp.getLundi().toYMD();
				nb = totalsemaine[i];
				txt += '<th scope="col">' + nb + '</th>';
				datetemp.setDate(datetemp.getDate() + 7);
			}
		}
	}
	txt += '</tr>';
	txt += '</table>';


	// affichage du tableau si l'utlisateur a coché la case et gestion des entêtes triables + css
	if (inputtableau){
		divresultats.html(txt);
		divresultats.css('display', 'block');
		jQuery('#resultats table tr:nth-child(2n)').css('background', 'rgb(240, 240, 240)');
	}
	else{ // tableau pas coché
		divresultats.html('');
		divresultats.css('display', 'none');
	}


	// affichage du graphique si l'utlisateur a coché la case du graphique
	// la bibliothèque highcharts est définie dans le fichier script/highcharts.js 
	if (inputgraphique){
		// affiche un graphique de type "ligne" si c'est l'évolution sur une période
		if (!inputtoutesdates && (inputhebdo || inputmensuel || inputannuel)){
			divresgraphique.highcharts({  
	            title: {text: 'Nombres d\'articles par élève', x: -20 },//-20 : center
	            xAxis: {categories: labelx},
	            yAxis: {title: {text: 'Nombres d\'articles par élève' }, plotLines: [{value: 0, width: 1, color: '#808080' }], min: 0},
	            legend: {layout: 'vertical', align: 'right', verticalAlign: 'middle', borderWidth: 0 },
	            colors: lescouleurs,
	            series: dataseries
	        });			
		}
		// affiche un graphique de type "histogramme" si c'est pour toutes les dates ou dates filtrées sans regroupement
		else{
			divresgraphique.highcharts({
		        chart: {defaultSeriesType: 'column'},
		        title: {text: 'Nombre d\'articles par élève', x: -20 },//-20 : center
		        xAxis: {categories: labelx},
		        yAxis: {title: {text: 'Nombres d\'articles par élève' }, plotLines: [{value: 0, width: 1, color: '#808080' }], min: 0},
		        legend: {enabled: false},
		        series: [{name: 'Nombre', data: dataseries}]
		    });	    
		}
	}
	// vide le div si l'utlisateur n'a coché la case du graphique
	else{
		divresgraphique.html('')
	}
}